@@ -1,6 +1,6 @@
 # configlet.py -- Python module for configlet 
 
-# $Progeny: configlet.py,v 1.118 2002/03/26 06:30:15 jlicquia Exp $
+# $Progeny$
 
 # Copyright (C) 2001  Progeny Linux Systems, Inc.
 # AUTHORS: Jeff Licquia <jlicquia@progeny.com>
@@ -32,6 +32,9 @@ import time
 import types
 import traceback
 
+# Whenever making incompatible changes to the API, increment this.
+api_version = 2
+
 configlet_classes = {}
 configlet_global_namespaces = {}
 started_configlets = {}
@@ -58,17 +61,14 @@ def _debconf_communicate(package, s):
 
 def load_gnome():
     global gnome_loaded
+    global pygtk
     global gtk
-    global libglade
 
     if not gnome_loaded:
-        #from gtk import *
-        #from libglade import *
+        import pygtk
+        pygtk.require("2.0")
         import gtk
-        import libglade
-
-        # Do we need this?  It would be really nice if we didn't.
-        #from gnome.ui import *
+        import gtk.glade
 
         gnome_loaded = 1
 
@@ -271,6 +271,38 @@ class SimplePrivilegedRunner:
             raise RuntimeError, "Command %s failed to execute, status %X" \
                   % (command, retval)
 
+# Privileged run interface based on gksu.
+
+class GksuPrivilegedRunner:
+    "This is a privileged runner class that uses xsu."
+
+    def run(self, command):
+        "Run the given command with privilege."
+
+        message = "Please type the root password to gain access."
+        title = "Configlet Root Access"
+        
+        debug("GksuPrivilegedRunner", "running %s" % (command,))
+
+        if os.getuid() == 0:
+            retval = os.system(command)
+        else:
+            temp_path = make_temp_dir()
+
+            script = open("%s/script" % (temp_path,), "w")
+            script.write("%s\n" % (command,))
+            script.close()
+
+            retval = os.system("gksu -u root -m '%s' -t '%s' '/bin/sh %s/script'"
+                               % (message, title, temp_path))
+
+            os.unlink("%s/script" % (temp_path,))
+            os.rmdir(temp_path)
+
+        if retval:
+            raise RuntimeError, "Command %s failed, exit status %X" \
+                  % (command, retval)
+
 # Privileged run interface based on gnome-sudo
 
 class GnomeSudoPrivilegedRunner:
@@ -544,7 +576,14 @@ class Configlet:
 
     def __init__(self, attr):
         "Initialize the configlet located in the provided directory."
-        
+
+        global api_version
+
+        if not attr.has_key("api_version"):
+            raise RuntimeError, "configlet does not support API versioning"
+        if attr["api_version"] != api_version:
+            raise RuntimeError, "API version mismatch"
+
         self.attributes = attr
         self.dirpath = self.attributes["dir"]
         self.can_assign_packages = 1
@@ -555,7 +594,7 @@ class Configlet:
                 path_components = os.path.split(path_components[0])
                 self.attributes["name"] = path_components[1]
 
-        self.priv_runner = GnomeSudoPrivilegedRunner()
+        self.priv_runner = GksuPrivilegedRunner()
 
     ###################################################################
     # Methods the inheriting configlet should (or may) override
@@ -583,7 +622,33 @@ class Configlet:
 
         load_gnome()
 
-        self.wtree = libglade.GladeXML("%s/main.glade" % (self.dirpath,))
+        # Now that we're in GNOME 2 land, we need to look at the
+        # glade file and determine if it's old-style or new-style.
+        # Old files should be run through the conversion script before
+        # loading them.
+
+        glade_file_path = "%s/main.glade" % (self.dirpath,)
+        glade_fo = open(glade_file_path)
+        is_gnome_2 = False
+        for line in glade_fo.readlines():
+            if string.find(line, "glade-2.0.dtd") != -1:
+                is_gnome_2 = True
+                break
+        glade_fo.close()
+
+        if not is_gnome_2:
+            temp_path = make_temp_dir()
+            load_file_path = "%s/main.glade" % (temp_path,)
+            os.system("libglade-convert %s > %s"
+                      % (glade_file_path, load_file_path))
+        else:
+            load_file_path = glade_file_path
+
+        self.wtree = gtk.glade.XML(load_file_path)
+
+        if not is_gnome_2:
+            os.unlink(load_file_path)
+            os.rmdir(temp_path)
 
         self.widgets = {}
         for pagename in self.get_page_names():
